
#include <math.h>
#include <iostream>
#include <conio.h>
#include <vector>
#include <string>
#include "svg.h"

using namespace std;

void
svg_begin(double width, double height)
{
    cout << "<?xml version='1.0' encoding='UTF-8'?>\n";
    cout << "<svg ";
    cout << "width='" << width << "' ";
    cout << "height='" << height << "' ";
    cout << "viewBox='0 0 " << width << " " << height << "' ";
    cout << "xmlns='http://www.w3.org/2000/svg'>\n";
}


void
svg_end()
{
    cout << "</svg>\n";
}

void
svg_text(double left, double baseline, string text)
{
    cout << "<text x='" << left << "' y='" << baseline << "'>" << text << "</text>";
}

void
svg_rect(double x, double y, double width, double height, string stroke = "black", string fill = "black")
{
    cout << "<rect x='"<<x<<"' y='"<<y<<"' width='"<<width<<"' height='"<<height<<"' stroke='"<<stroke<<"' fill='"<<fill<<"' />";

}



void
show_histogram_svg(const vector<size_t>& bins, const vector<double>& borders, size_t bin_count)
{
    const auto IMAGE_WIDTH = 400;
    const auto IMAGE_HEIGHT = 300;
    const auto TEXT_LEFT = 20;
    const auto TEXT_BASELINE = 20;
    const auto TEXT_WIDTH = 50;
    const auto BIN_HEIGHT = 30;
    const auto BLOCK_WIDTH = 10;
    const auto COLOUR1 = "black";
    const auto COLOUR2 = "green";
    const auto MAX_WIDTH = IMAGE_WIDTH-TEXT_WIDTH;
    size_t border = 0;
    size_t number_of_blocks;

    svg_begin(IMAGE_WIDTH,IMAGE_HEIGHT);
    size_t max_bin = *max_element(bins.begin(), bins.end());

    double top = 0;
    double max_count = bins[0];
    for (size_t i = 0; i < bins.size(); i++)
    {
        if (max_count<bins[i])
        {
            max_count=bins[i];
        }
    }

    for (size_t bin : bins)
    {
        number_of_blocks = bin;
        if ((max_bin * BLOCK_WIDTH) > (IMAGE_WIDTH - TEXT_WIDTH)){
            number_of_blocks = ((IMAGE_WIDTH - TEXT_WIDTH)/10) * (static_cast<double>(bin) / max_bin);
        }
        const double bin_width = BLOCK_WIDTH * number_of_blocks;
        svg_text(TEXT_LEFT, top + TEXT_BASELINE, to_string(bin));
        svg_rect(TEXT_WIDTH, top, bin_width, BIN_HEIGHT, COLOUR1, COLOUR2);
        top += BIN_HEIGHT;
        if (border < bin_count - 1){
            top += BIN_HEIGHT;
            svg_text(TEXT_LEFT / 2, top + TEXT_BASELINE , to_string(borders[border]));
            border += 1;
            top += BIN_HEIGHT;
        }
    }

    svg_end();
}
